import React, { lazy, Suspense } from 'react';
import PropTypes from 'prop-types';
import { getURLParam } from 'gatsby-core-theme/src/helpers/getters';
import Module from '~molecules/module';
import styles from './main.module.scss';
import keygen from '~helpers/keygen';
import settings from '../../../constants/settings.mjs';
import AuthorBoxComponent from '../../atoms/author-box';
import { isSearchPath } from '~helpers/isSearchPath';

const Main = ({
  section = {},
  pageContext = {},
  serverData = {},
  exclOperator = false,
  exclNewsletter = false,
}) => {
  const { page, allMarkets } = pageContext;
  const siteName = process.env.GATSBY_SITE_NAME;
  const showNewsletter =
    !(
      settings.newsletter[siteName]?.filter_pages?.includes('*') ||
      settings.newsletter[siteName]?.filter_pages?.includes(page?.relation_type) ||
      settings.newsletter.default?.filter_pages?.includes(page?.relation_type) ||
      exclNewsletter
    ) && !process.env?.HIDE_NEWSLETTER;
  const pathPrefixes = isSearchPath(allMarkets, page.path);

  const SearchPage = pathPrefixes ? lazy(() => import(`~pages/search`)) : null;
  const HtmlSitemap = pageContext.sitemapData ? lazy(() => import(`~atoms/sitemap`)) : null;
  const AuthorBox = page.author_id;
  const NotFound = page?.path?.includes('404') ? lazy(() => import(`~atoms/not-found`)) : null;
  const Newsletter = showNewsletter ? lazy(() => import('~molecules/newsletter')) : null;
  const NewsletterSuccess =
    showNewsletter && typeof window !== 'undefined' && Boolean(getURLParam('subscribed'))
      ? lazy(() => import(`../newsletter/success`))
      : null;

  return (
    <main className={styles.modulePage || ''}>
      {section.modules &&
        section.modules.map((module, index) => (
          <Module
            key={keygen()}
            module={module}
            exclOperator={exclOperator}
            page={page}
            pageContext={pageContext}
            serverData={serverData}
            modulePosition={index}
          />
        ))}
      {NotFound && (
        <Suspense fallback={<></>}>
          <NotFound pageContext={pageContext} />
        </Suspense>
      )}
      {HtmlSitemap && (
        <Suspense fallback={<></>}>
          <HtmlSitemap pageContext={pageContext} />
        </Suspense>
      )}
      {AuthorBox && (
        <Suspense fallback={<></>}>
          <AuthorBoxComponent
            author={page.author}
            date={page.updated_at ? page.updated_at : page.created_at}
            readingTime={page.reading_time}
            reviewer={page.reviewer_id && page.reviewer}
          />
        </Suspense>
      )}
      {SearchPage && (
        <Suspense fallback={<></>}>
          <SearchPage page={page} />
        </Suspense>
      )}
      {Newsletter && (
        <Suspense fallback={<></>}>
          <Newsletter page={page} isSticky={false} />
        </Suspense>
      )}
      {NewsletterSuccess && (
        <Suspense fallback={<></>}>
          <NewsletterSuccess />
        </Suspense>
      )}
    </main>
  );
};

Main.propTypes = {
  section: PropTypes.shape({
    modules: PropTypes.arrayOf(
      PropTypes.shape({
        name: PropTypes.string,
      })
    ),
  }),
  page: PropTypes.shape({
    path: PropTypes.string,
  }),
  pageContext: PropTypes.shape({}),
  serverData: PropTypes.shape({}),
  exclOperator: PropTypes.bool,
  exclNewsletter: PropTypes.bool,
};

export default Main;
